//############################################################################
// ~ International Laser Display Association ~
// ~     ILDA File Format Driver in C++      ~
// ~             Built for EZFB              ~
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//############################################################################
#ifndef __EZ_ILDA_DEFINITIONS__
#define __EZ_ILDA_DEFINITIONS__

//############################################################################
#include "ez_ilda_header.hpp"
#include "ez_ilda_frame_set.hpp"
#include "ez_palette_set.hpp"
#include "ez_bmp.hpp"

//############################################################################
class ez_ilda : public ez_ilda_frame_set, public ez_palette_set
{
public:
       
    ez_ilda()
               : ez_ilda_frame_set()
               , ez_palette_set   ()
                {}
    //------------------------------------------------------------------------
    ez_ilda(const ez_ilda_frame_set& frames)
               : ez_ilda_frame_set()
               , ez_palette_set   ()  
                {
                    ez_ilda_frame_set::insert(   ez_ilda_frame_set::end()
                                               , frames.begin()
                                               , frames.end()
                                             );
                    renumber();
                }
    //------------------------------------------------------------------------
virtual
   ~ez_ilda()   {}
    //------------------------------------------------------------------------
    ez_ilda_frame& operator [] (int index) // does not set frame_index
                    {
                        if(number_of_frames())
                        {
                            if(index >= number_of_frames())
                                index = 0;
                            if(index < 0)
                                index = number_of_frames() - 1;
                            return ez_ilda_frame_set::at(index);
                        }
                        return null_frame;
                    }
    //------------------------------------------------------------------------
    ez_ilda& operator += (const ez_ilda_frame_set& frames)
                    {
                        ez_ilda_frame_set::insert(ez_ilda_frame_set::end(), frames.begin(), frames.end());
                        renumber();
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_ilda& operator  = (const ez_ilda_frame_set& frames)
                    {
                        ez_ilda_frame_set::clear();
                        ez_ilda_frame_set::insert(ez_ilda_frame_set::end(), frames.begin(), frames.end());
                        renumber();
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_ilda& operator += (const ez_ilda_frame& frame)
                    {
                        ez_ilda_frame_set::push_back(frame);
                        renumber();
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_ilda& operator += (const ez_palette_set& palettes)
                    {
                        ez_palette_set::insert(ez_palette_set::end(), palettes.begin(), palettes.end());
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_ilda& operator += (const ez_palette& palette)
                    {
                        ez_palette_set::push_back(palette);
                        return *this;
                    }  
    //------------------------------------------------------------------------
    void           clear_frames()
                        {
                            ez_ilda_frame_set::clear();
                            return;
                        }
    //------------------------------------------------------------------------
    void           clear_palettes()
                        {
                            ez_palette_set::clear();
                            return;
                        }
    //------------------------------------------------------------------------
    void           sync_rgb_and_palette();
    //------------------------------------------------------------------------
};

//############################################################################
#endif

//############################################################################
//////////////////////////////////////////////////////////////////////////////
//############################################################################
