//############################################################################
// ~ International Laser Display Association ~
// ~     ILDA File Format Driver in C++      ~
// ~             Built for EZFB              ~
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//############################################################################
#ifndef __EZ_ILDA_FRAME_DEFINITIONS__
#define __EZ_ILDA_FRAME_DEFINITIONS__

//############################################################################
#include "ez_ilda_segment.hpp"
#include "ez_real_segment.hpp"

//############################################################################
class ez_ilda_frame : public ez_ilda_header, public ez_ilda_segment
{
public:
    //------------------------------------------------------------------------
    ez_ilda_frame()
                 : ez_ilda_header     (     ),
                   point_select_start (0    ),
                   point_select_end   (0    ),
                   is_selected        (false),
                   is_unique          (true )                     {}
    //------------------------------------------------------------------------
    ez_ilda_frame(const ez_ilda_frame& frame)
                 : ez_ilda_header     ((ez_ilda_header )frame),
                   ez_ilda_segment    ((ez_ilda_segment)frame),
                   point_select_start (0                     ),
                   point_select_end   (0                     ),
                   is_selected        (false                 ),
                   is_unique          (frame.is_unique       )    {}
   //-------------------------------------------------------------------------
    ez_ilda_frame(const int& palette_index)
                 : ez_ilda_header     (             ),
                   ez_ilda_segment    (palette_index),
                   point_select_start (0            ),
                   point_select_end   (0            ),
                   is_selected        (false        ),
                   is_unique          (true         )             {}
    //------------------------------------------------------------------------
    ez_ilda_frame(const int& palette_index, int not_used)
                 : ez_ilda_header     (                       ),
                   ez_ilda_segment    (palette_index, not_used),
                   point_select_start (0                      ),
                   point_select_end   (0                      ),
                   is_selected        (false                  ),
                   is_unique          (true                   )   {}
    //------------------------------------------------------------------------
    ez_ilda_frame(const ez_ilda_segment& segment, const ez_ilda_header& header)
                 : ez_ilda_header     (header ),
                   ez_ilda_segment    (segment),
                   point_select_start (0      ),
                   point_select_end   (0      ),
                   is_selected        (false  ),
                   is_unique          (true   )                   {}
    //------------------------------------------------------------------------
    ez_ilda_frame(const char&     format  ,
                  const string&   name    ,
                  const string&   owner   ,
                  const u_short&  qauntity,
                  const u_short&  identity,
                  const u_short&  total   ,
                  const char&     scanner
                 )
                 : ez_ilda_header     (format  ,
                                       name    ,
                                       owner   ,
                                       quantity,
                                       identity,
                                       total   ,
                                       scanner ,
                                       0
                                      ),
                   point_select_start (0    ),
                   point_select_end   (0    ),
                   is_selected        (false),
                   is_unique          (true )                   {}
    //------------------------------------------------------------------------
    ez_ilda_frame(ez_ilda_point(*F)(int, int), int points_per_frame, int frame_index);
    //------------------------------------------------------------------------
virtual
   ~ez_ilda_frame()                                             {}
    //------------------------------------------------------------------------
    void validate()
            {
                quantity = (u_short)number_of_points();                                 
                if(point_select_start >= quantity)
                    point_select_start = 0;
                if(point_select_end >= quantity)
                    point_select_end = 0;
            }
    //------------------------------------------------------------------------
    bool is_2D()
            {
                if(ez_ilda_segment::is_2D())
                    format = EZ_ILDA_2D_VECTOR;
                else
                    format = EZ_ILDA_3D_VECTOR;
                return (bool)format; // 3D = 0, 2D = 1
            }
    //------------------------------------------------------------------------
    bool is_point_selected(int point_index) const
            {
                if(point_index >= 0 && point_index < number_of_points())
                {
                    if     (    point_select_start == point_select_end
                             && point_index        == point_select_start
                           )
                          return true;
                    else if(    point_select_start <  point_select_end
                             && point_index        >= point_select_start
                             && point_index        <= point_select_end
                           )
                          return true;
                    else if(     point_index <= point_select_start
                              && point_index >= point_select_end
                           )
                          return true;
                }
                return false;
            }
    //------------------------------------------------------------------------
    ez_ilda_segment selected_segment() const
                        {
                            int             i;
                            ez_ilda_segment segment;
                            for(i = 0; i < (int)size(); i++)
                                if(is_point_selected(i))
                                    segment += at(i);
                            return segment;
                        }
    //------------------------------------------------------------------------
    ez_ilda_frame& operator =  (const ez_ilda_frame& frame)
                    {
                        format        = frame.format       ;
                        quantity      = frame.quantity     ;
                        identity      = frame.identity     ;
                        total         = frame.total        ;
                        scanner       = frame.scanner      ;
                        future        = 0                  ;
                        error         = EZ_ILDA_OK         ;
                        palette_index = frame.palette_index;
                        clear();
                        insert(begin(), frame.begin(), frame.end());
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_ilda_frame& operator =  (const ez_ilda_header& header)
                    {
                        format    = header.format   ;
                        name      = header.name     ;
                        owner     = header.owner    ;
                        quantity  = header.quantity ;
                        identity  = header.identity ;
                        total     = header.total    ;
                        scanner   = header.scanner  ;
                        future    = 0               ;
                        error     = EZ_ILDA_OK      ;
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_ilda_frame& operator =  (const ez_ilda_segment& segment)
                    {
                        clear();
                        palette_index = segment.palette_index;
                        insert(begin(), segment.begin(), segment.end());
                        format    = is_2D();
                        quantity  = (u_short)number_of_points();
                        error     = EZ_ILDA_OK;
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_ilda_frame& operator += (const ez_ilda_frame& frame)
                    {
                        insert(end(), frame.begin(), frame.end());
                        quantity = (u_short)number_of_points();
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_ilda_frame& operator += (const ez_ilda_segment& segment)
                    {
                        insert(end(), segment.begin(), segment.end());
                        quantity = (u_short)number_of_points();
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_ilda_frame& operator += (const ez_ilda_point& point)
                    {
                        push_back(point);
                        quantity = (u_short)number_of_points();
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_ilda_frame  operator +  (const ez_ilda_frame& frame) const
                    {
                        ez_ilda_frame sum(*this);
                        sum += frame;
                        return sum;
                    }
    //------------------------------------------------------------------------
    ez_ilda_frame& add_new_point    (int point_index);
    //------------------------------------------------------------------------
    ez_ilda_frame& break_segment    ();    
    //------------------------------------------------------------------------
    ez_ilda_frame& connect_the_dots ();
    //------------------------------------------------------------------------
    void minimize_blank_points()
            {
                ez_ilda_segment::minimize_blank_points();
                validate();
                return;
            }
    //------------------------------------------------------------------------
    void remove_dwell_points()
            {
                ez_ilda_segment::remove_dwell_points();
                validate();
                return;
            }
    //------------------------------------------------------------------------
    void minimize_vectors()
            {
                ez_ilda_segment::minimize_vectors();
                validate();
                return;
            }
    //------------------------------------------------------------------------
    void minimize_points()
            {
                ez_ilda_segment::minimize_points();
                validate();
                return;
            }
    //------------------------------------------------------------------------
    ez_ilda_frame& reorder_segments()
                    {
                        ez_ilda_segment::reorder_segments();
                        validate();
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_ilda_frame& fracture_segments()
                    {
                        ez_ilda_segment::fracture_segments();
                        validate();
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_ilda_frame& bond_segments()
                    {
                        ez_ilda_segment::bond_segments();
                        validate();
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_ilda_frame& reverse()
                    {
                        ez_ilda_segment::reverse();
                        validate();
                        return *this;
                    }
    //------------------------------------------------------------------------
    void add_dwell_points()
            {
                ez_ilda_segment::add_dwell_points();
                validate();
                return;
            }
    //------------------------------------------------------------------------
    void add_span_points()
            {
                ez_ilda_segment::add_span_points();
                validate();
                return;
            }
    //------------------------------------------------------------------------
    ez_ilda_frame& remove_point(int point_index)
                    {
                        ez_ilda_segment::remove_point(point_index);
                        validate();
                        return *this;
                    }
    //------------------------------------------------------------------------
    void  increment_point_select_start()
            {
                point_select_start++;
                if(point_select_start >= quantity)
                    point_select_start = 0;
            }
    //------------------------------------------------------------------------
    void  deccrement_point_select_start()
            {
                point_select_start--;
                if(point_select_start < 0)
                    point_select_start = quantity - 1;
            }
    //------------------------------------------------------------------------
    void  increment_point_select_end()
            {
                point_select_end++;
                if(point_select_end >= quantity)
                    point_select_end = 0;
            }
    //------------------------------------------------------------------------
    void  deccrement_point_select_end()
            {
                point_select_end--;
                if(point_select_end < 0)
                    point_select_end = quantity - 1;
            }
    //------------------------------------------------------------------------
    void save_as_ilda            (string file                               );
    void optimize_points         (                                          );
    int  from_ifstream_ilda_f3   (ifstream& in, const ez_ilda_header& header);
    int  from_ifstream_ilda      (ifstream& in, const ez_ilda_header& header);
    int  from_ifstream_dxf       (ifstream& in, const ez_ilda_header& header);
    int  from_ifstream_shp       (ifstream& in, const ez_ilda_header& header);
    int  move_selection          (ez_3D_float d                             );
    //------------------------------------------------------------------------
    int scale_selection                        (ez_3D_float m               );
    int scale_selection_around_point           (ez_3D_float d, ez_3D_float p);
    int scale_selection_around_arbitrary_point (ez_3D_float d               );
    //------------------------------------------------------------------------
    int rotate_selection                       (ez_3D_float a               );
    int rotate_selection_around_point          (ez_3D_float a, ez_3D_float p);
    int rotate_selection_around_arbitrary_point(ez_3D_float a               );
    //------------------------------------------------------------------------
    void next_segment_select           ();
    void next_segment_select_start     ();
    void next_segment_select_end       ();
    void previous_segment_select       ();
    void previous_segment_select_start ();
    void previous_segment_select_end   ();
    //------------------------------------------------------------------------
    void set_points_color_index   (                                 )       ;
    void blank_points             (                                 )       ;
    void unblank_points           (                                 )       ;
    void black_points             (                                 )       ;
    void unblack_points           (                                 )       ;
    void convert_black_to_blank   (                                 )       ;
    void to_ofstream_ilda         (ofstream& out                    )       ;
    void save_as_wave             (string file_name                 ) const ;
    void to_ofstream_wave         (ofstream& out                    ) const ;
    void render_in_ezfb_bitmap    (string ui_message = ""           ) const ;
    void to_ofstream_code         (ofstream& out, const char* indent) const ;
    void save_as_code             (string file_name                 ) const ;
    void save_as_dxf              (string file_name                 ) const ;
    void save_as_bmp              (string file_name                 ) const ;
    void stats                    (bool show_cursors                ) const ;
    void display_stats            (                                 ) const ;
    //------------------------------------------------------------------------
    ez_ilda_segment  transition_to_next_frame;

    int              point_select_start,
                     point_select_end  ;

    bool             is_selected,
                     is_unique  ;
};
//############################################################################
ez_ilda_frame blank_frame();
ez_ilda_frame NULL_frame ();

//############################################################################
#endif

//############################################################################
//////////////////////////////////////////////////////////////////////////////
//############################################################################
