//############################################################################
// ~ International Laser Display Association ~
// ~     ILDA File Format Driver in C++      ~
// ~             Built for EZFB              ~
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//############################################################################
#ifndef __EZ_ILDA_GUI_DEFINITIONS__
#define __EZ_ILDA_GUI_DEFINITIONS__

//############################################################################
#include "ez_ilda_app.hpp"

//############################################################################
class ez_ilda_gui : public ez_ilda_app
{
public:
    ez_ilda_gui(u_int xres, u_int yres);
    //------------------------------------------------------------------------
virtual
   ~ez_ilda_gui();
    //------------------------------------------------------------------------
    void           load_cfg_file();
    void           save_cfg_file();
    //------------------------------------------------------------------------
    ez_ilda_gui& operator = (const ez_ilda_frame_set& frames)
                    {
                        ez_ilda_frame_set::clear();
                        ez_ilda_frame_set::insert(ez_ilda_frame_set::end(), frames.begin(), frames.end());
                        renumber();
                        return *this;
                    }
    //------------------------------------------------------------------------
    void           display_stats() const
                        {
                            ez_ilda_frame_set::display_stats();
                            ez_palette_set::display_stats();
                            return;
                        }
    //------------------------------------------------------------------------
    void           simplify_view_angle()
                        {
                            while(view_angle.x < 0)
                                view_angle.x += two_pi;
                            while(view_angle.x > two_pi)
                                view_angle.x -= two_pi;

                            while(view_angle.y < 0)
                                view_angle.y += two_pi;
                            while(view_angle.y > two_pi)
                                view_angle.y -= two_pi;

                            while(view_angle.z < 0)
                                view_angle.z += two_pi;
                            while(view_angle.z > two_pi)
                                view_angle.z -= two_pi;
                        }
    //------------------------------------------------------------------------
    void           display                         (string ui_message = "" );
    string         display_prompt_and_echo_name    (string prompt          );
    string         display_prompt_and_echo_string  (string prompt          );
    float          display_prompt_and_echo_float   (string prompt          );
    int            display_prompt_and_echo_int     (string prompt          );
    bool           display_prompt_and_echo_bool    (string prompt          );
    void           display_program_menu            ();
    void           display_please_wait             ();
    void           display_in_file_type_menu       ();
    void           display_in_ild_action_menu      ();
    void           display_in_dxf_action_menu      ();
    void           display_in_wave_action_menu     ();
    void           display_in_bmp_action_menu      ();
    void           display_in_pal_action_menu      ();
    void           display_out_file_type_menu      ();
    void           display_settings_menu           ();
    void           display_veiw_menu               ();    
    void           display_visuals_menu            ();
    void           display_palette_tranz_menu      ();
    void           display_frame_set_tranz_menu    ();
    void           display_frame_tranz_menu        ();
    void           display_draw_color_blank_menu   ();
    void           display_move_scale_rotate_menu  ();
    void           display_wave_utilities_menu     ();
    void           display_wave_show_settings_menu ();
    void           display_wave_stats              (string file_name       );
    void           display_wave                    (string file_name       );
    void           display_wave_offsets            ();
    void           display_file_list               (string directory       );
    void           display_directory_list          (string directory       );
    //------------------------------------------------------------------------
    void           load_background_bitmap          (struct ezfb_bitmap* bmp);
    void           load_background_bitmap          (string file            );
    void           clear_background_bitmap         ();
    void           recolor_background              ();
    //------------------------------------------------------------------------
    void           display_frame_effects           ();
    void           display_frame_set_effects       ();
    //------------------------------------------------------------------------
struct
    ezfb_bitmap    bmp                    , // a memory clone of screen
                   background             ;
    //------------------------------------------------------------------------
    string         bg_file_name           ;                          
    //------------------------------------------------------------------------
    ez_color       bg_color               ;
    //------------------------------------------------------------------------
    int            font_size_factor       ,
                   move_points_per_key    ;
    //------------------------------------------------------------------------
    ez_3D_float    arbitrary_point        ,
                   view_angle             ,
                   view_scale             ,
                   view_offset            ;
    //------------------------------------------------------------------------
    double         rotate_radians_per_key ; // in radians!
    //------------------------------------------------------------------------
    bool           show_points            ,
                   show_blanking          ,
                   show_cursors           ,
                   show_palettes          ,
                   show_stats             ,
                   show_fixed_bounds      ,
                   show_fixed_origin      ,
                   show_floating_axis     ,
                   show_floating_bounds   ,
                   show_ui_message        ,
                   show_arbitrary_point   ,
                   show_effects_generation,
                   loop_animation         ,
                   approximate_frame_rate ,
                   wave_show_points       ,
                   wave_show_blanks       ,
                   wave_real_time         ,
                   wave_show_inverted     ,
                   wave_clean_screen      ,
                   wave_loop              ,
                   process_exception      ;
    //------------------------------------------------------------------------
};

//###################################################################################
extern ez_ilda_gui* EZI;    // declaration of the one and only!
extern SDL_Surface* screen; // declaration of an SDL graphics window
extern SDL_Event    event;

//############################################################################
#endif

//############################################################################
//////////////////////////////////////////////////////////////////////////////
//############################################################################
