//############################################################################
// ~ International Laser Display Association ~
// ~     ILDA File Format Driver in C++      ~
// ~             Built for EZFB              ~
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//############################################################################
#ifndef __EZ_ILDA_UTILITY_DEFINITIONS__
#define __EZ_ILDA_UTILITY_DEFINITIONS__

//############################################################################
#include "ez_ilda_point.hpp"

//############################################################################
void    undo_wave_temp(ifstream& in, ofstream& out, string file_name);

//############################################################################
int     distance_points(const ez_ilda_point p1, const ez_ilda_point p2);
int     dwell_points   (double angle);
u_int   next_hue       ();

//############################################################################
inline ez_3D_float rotate_point(ez_3D_float p, ez_3D_float a)
{
    ez_3D_float   sin_a(sin(a.x), sin(a.y), sin(a.z)),
                  cos_a(cos(a.x), cos(a.y), cos(a.z)),
                  rotated;
                    
    rotated.y =       p.y * cos_a.x -  p.z * sin_a.x;
    rotated.z =       p.z * cos_a.x +  p.y * sin_a.x; // rotate around x
    rotated.z = rotated.z * cos_a.y -  p.x * sin_a.y;
    rotated.x =       p.x * cos_a.y + (p.z * cos_a.x +  p.y * sin_a.x) * sin_a.y; // rotate around y
    rotated.x = rotated.x * cos_a.z - (p.y * cos_a.x -  p.z * sin_a.x) * sin_a.z;
    rotated.y = rotated.y * cos_a.z + (p.x * cos_a.y + (p.z * cos_a.x +  p.y * sin_a.x) * sin_a.y) * sin_a.z; // rotate around z
    return rotated;
}

//############################################################################
inline ez_3D_float rotate_point_around_x(ez_3D_float p, float a)
{
    float         sin_a(sin(a)),
                  cos_a(cos(a));
    ez_3D_float   rotated(p);

    rotated.y = p.y * cos_a -  p.z * sin_a;
    rotated.z = p.z * cos_a +  p.y * sin_a; // rotate around x
    return rotated;
}

//############################################################################
inline ez_3D_float rotate_point_around_y(ez_3D_float p, float a)
{
    float         sin_a(sin(a)),
                  cos_a(cos(a));
    ez_3D_float   rotated(p);

    rotated.x = p.x * cos_a -  p.z * sin_a;
    rotated.z = p.z * cos_a +  p.x * sin_a; // rotate around y
    return rotated;
}

//############################################################################
inline ez_3D_float rotate_point_around_z(ez_3D_float p, float a)
{
    float         sin_a(sin(a)),
                  cos_a(cos(a));
    ez_3D_float   rotated(p);

    rotated.x = p.x * cos_a -  p.y * sin_a;
    rotated.y = p.y * cos_a +  p.x * sin_a; // rotate around z
    return rotated;
}

//############################################################################
inline ez_3D_float rotate_point_around_point(ez_3D_float p1, ez_3D_float p2, ez_3D_float a)
{
    return rotate_point(p1 - p2, a) + p2;
}

//############################################################################
inline ez_3D_float rotate_point_around_point_x(ez_3D_float p1, ez_3D_float p2, float a)
{
    return rotate_point_around_x(p1 - p2, a) + p2;
}

//############################################################################
inline ez_3D_float rotate_point_around_point_y(ez_3D_float p1, ez_3D_float p2, float a)
{
    return rotate_point_around_y(p1 - p2, a) + p2;
}

//############################################################################
inline ez_3D_float rotate_point_around_point_z(ez_3D_float p1, ez_3D_float p2, float a)
{
    return rotate_point_around_z(p1 - p2, a) + p2;
}

//############################################################################
inline ez_3D_float scale_point_around_point(ez_3D_float p1, ez_3D_float  p2, ez_3D_float m)
{
    return (m * (p1 - p2)) + p2;
}

//############################################################################
inline int ez_ilda_bounds_check( const ez_3D_float& f,
                                 const ez_3D_short& max,
                                 const ez_3D_short& min
                               )
{
    if(f.x > max.x)  return EZ_OUT_POS_X;
    if(f.x < min.x)  return EZ_OUT_NEG_X;
    if(f.y > max.y)  return EZ_OUT_POS_Y;
    if(f.y < min.y)  return EZ_OUT_NEG_Y;
    if(f.z > max.z)  return EZ_OUT_POS_Z;
    if(f.z < min.z)  return EZ_OUT_NEG_Z;
    return EZ_IN_BOUNDS;
}

//############################################################################
inline int ez_ilda_bounds_check(const ez_3D_float& f, const int& space)
{
    ez_3D_short max, min;
    switch(space)
    {
        case EZ_ILDA_CUBE          : max =  32767 ;
                                     min = -32768 ;
                                     break        ;
        //--------------------------------------------------------------------
        case EZ_ILDA_SPHERE        : max =  23169 ;
                                     min = -23169 ;
                                     break        ;
        //--------------------------------------------------------------------
        case EZ_ILDA_7_8THS_CUBE   : max =  28672 ;
                                     min = -28672 ;
                                     break        ;
        //--------------------------------------------------------------------
        case EZ_ILDA_7_8THS_SPHERE : max =  20274 ;
                                     min = -20274 ;
                                     break        ;
        //--------------------------------------------------------------------
    }
    return ez_ilda_bounds_check(f, max, min);
}

//############################################################################
inline void undo_wave_temp(ifstream &in, ofstream &out, string file_name)
{
    if(in.is_open())
        in.close();
    if(out.is_open())
        out.close();
    remove((EZ_ILDA_WAVE_SHARE + file_name).c_str());
    rename(  (EZ_ILDA_WAVE_SHARE + "__TEMP__.wav").c_str()
           , (EZ_ILDA_WAVE_SHARE + file_name).c_str()
          );        
    return;
}

//############################################################################
inline string ez_wave_signal_id_to_name(short signal_id)
{
    switch(signal_id)
    {
        default:
        case EZ_SIGNAL_UNDEFINED:
             return "UNDEFINED_SIGNAL";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_X_POSITION:
             return "X_POSITION";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_Y_POSITION:
             return "Y_POSITION";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_Z_POSITION:
             return "Z_POSITION";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_DX_POSITION:
             return "DX_POSITION";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_DY_POSITION:
             return "DY_POSITION";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_BEAM_WIDTH:
             return "BEAM_WIDTH";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_RED_TTL:
             return "RED_TTL";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_RED_ANALOG:
             return "RED_ANALOG";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_YELLOW_TTL:
             return "YELLOW_TTL";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_YELLOW_ANALOG:
             return "YELLOW_ANALOG";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_GREEN_TTL:
             return "GREEN_TTL";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_GREEN_ANALOG:
             return "GREEN_ANALOG";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_BLUE_TTL:
             return "BLUE_TTL";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_BLUE_ANALOG:
             return "BLUE_ANALOG";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_VIOLET_TTL:
             return "VIOLET_TTL";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_VIOLET_ANALOG:
             return "VIOLET_ANALOG";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_MONO_TTL:
             return "MONO_TTL";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_MONO_ANALOG:
             return "MONO_ANALOG";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_AUDIO_SMTPE:
             return "AUDIO_SMTPE";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_AUDIO_MONO:
             return "AUDIO_MONO";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_AUDIO_LEFT:
             return "AUDIO_LEFT";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_AUDIO_RIGHT:
             return "AUDIO_RIGHT";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_AUDIO_CENTER:
             return "AUDIO_CENTER";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_AUDIO_SUB:
             return "AUDIO_SUB";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_AUDIO_S_LEFT:
             return "AUDIO_S_LEFT";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_AUDIO_S_RIGHT:
             return "AUDIO_S_RIGHT";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_AUDIO_R_LEFT:
             return "AUDIO_R_LEFT";
             break;
        //--------------------------------------------------------------------
        case EZ_SIGNAL_AUDIO_R_RIGHT:
             return "AUDIO_R_RIGHT";
             break;
        //--------------------------------------------------------------------
    }
    return "";
}

//############################################################################
inline string ez_wave_LSB_tag_to_name(short LSB_tag)
{
    switch(LSB_tag)
    {
        default:
        case EZ_LSB_NOT_USED:
             return "LSB_NOT_USED";
             break;
        //--------------------------------------------------------------------
        case EZ_LSB_BLANKING:
             return "LSB_BLANKING";
             break;
        //--------------------------------------------------------------------
        case EZ_LSB_END_OF_FRAME:
             return "LSB_END_OF_FRAME";
             break;
        //--------------------------------------------------------------------
        case EZ_LSB_UNIQUE_FRAME:
             return "LSB_UNIQUE_FRAME";
             break;
        //--------------------------------------------------------------------
        case EZ_LSB_UNIQUE_POINT:
             return "LSB_UNIQUE_POINT";
             break;
        //--------------------------------------------------------------------
    }
    return "";
}

//############################################################################
#endif

//############################################################################
//////////////////////////////////////////////////////////////////////////////
//############################################################################
